@@ -1,9 +1,24 @@
+# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+#
+# Licensed under the Apache License, Version 2.0 (the "License"). You
+# may not use this file except in compliance with the License. A copy of
+# the License is located at
+#
+# http://aws.amazon.com/apache2.0/
+#
+# or in the "license" file accompanying this file. This file is
+# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
+# ANY KIND, either express or implied. See the License for the specific
+# language governing permissions and limitations under the License.
 """Abstractions to interact with service models."""
 from collections import defaultdict
 
-from botocore.utils import CachedProperty
 from botocore.compat import OrderedDict
-
+from botocore.exceptions import (
+    MissingServiceIdError,
+    UndefinedModelAttributeError,
+)
+from botocore.utils import CachedProperty, hyphenize_service_id, instance_cache
 
 NOT_SET = object()
 
@@ -24,20 +39,52 @@ class InvalidShapeReferenceError(Exception):
     pass
 
 
-class UndefinedModelAttributeError(Exception):
-    pass
+class ServiceId(str):
+    def hyphenize(self):
+        return hyphenize_service_id(self)
 
 
-class Shape(object):
+class Shape:
     """Object representing a shape from the service model."""
+
     # To simplify serialization logic, all shape params that are
     # related to serialization are moved from the top level hash into
     # a 'serialization' hash.  This list below contains the names of all
     # the attributes that should be moved.
-    SERIALIZED_ATTRS = ['locationName', 'queryName', 'flattened', 'location',
-                        'payload', 'streaming', 'timestampFormat',
-                        'xmlNamespace', 'resultWrapper', 'xmlAttribute']
-    METADATA_ATTRS = ['required', 'min', 'max', 'sensitive', 'enum']
+    SERIALIZED_ATTRS = [
+        'locationName',
+        'queryName',
+        'flattened',
+        'location',
+        'payload',
+        'streaming',
+        'timestampFormat',
+        'xmlNamespace',
+        'resultWrapper',
+        'xmlAttribute',
+        'eventstream',
+        'event',
+        'eventheader',
+        'eventpayload',
+        'jsonvalue',
+        'timestampFormat',
+        'hostLabel',
+    ]
+    METADATA_ATTRS = [
+        'required',
+        'min',
+        'max',
+        'pattern',
+        'sensitive',
+        'enum',
+        'idempotencyToken',
+        'error',
+        'exception',
+        'endpointdiscoveryid',
+        'retryable',
+        'document',
+        'union',
+    ]
     MAP_TYPE = OrderedDict
 
     def __init__(self, shape_name, shape_model, shape_resolver=None):
@@ -90,6 +137,8 @@ class Shape(object):
             * xmlNamespace
             * resultWrapper
             * xmlAttribute
+            * jsonvalue
+            * timestampFormat
 
         :rtype: dict
         :return: Serialization information about the shape.
@@ -113,9 +162,13 @@ class Shape(object):
 
             * min
             * max
+            * pattern
             * enum
             * sensitive
             * required
+            * idempotencyToken
+            * document
+            * union
 
         :rtype: dict
         :return: Metadata about the shape.
@@ -143,14 +196,17 @@ class Shape(object):
         return self._shape_resolver.resolve_shape_ref(shape_ref)
 
     def __repr__(self):
-        return "<%s(%s)>" % (self.__class__.__name__,
-                             self.name)
+        return f"<{self.__class__.__name__}({self.name})>"
+
+    @property
+    def event_stream_name(self):
+        return None
 
 
 class StructureShape(Shape):
     @CachedProperty
     def members(self):
-        members = self._shape_model['members']
+        members = self._shape_model.get('members', self.MAP_TYPE())
         # The members dict looks like:
         #    'members': {
         #        'MemberName': {'shape': 'shapeName'},
@@ -162,6 +218,32 @@ class StructureShape(Shape):
             shape_members[name] = self._resolve_shape_ref(shape_ref)
         return shape_members
 
+    @CachedProperty
+    def event_stream_name(self):
+        for member_name, member in self.members.items():
+            if member.serialization.get('eventstream'):
+                return member_name
+        return None
+
+    @CachedProperty
+    def error_code(self):
+        if not self.metadata.get('exception', False):
+            return None
+        error_metadata = self.metadata.get("error", {})
+        code = error_metadata.get("code")
+        if code:
+            return code
+        # Use the exception name if there is no explicit code modeled
+        return self.name
+
+    @CachedProperty
+    def is_document_type(self):
+        return self.metadata.get('document', False)
+
+    @CachedProperty
+    def is_tagged_union(self):
+        return self.metadata.get('union', False)
+
 
 class ListShape(Shape):
     @CachedProperty
@@ -179,18 +261,18 @@ class MapShape(Shape):
         return self._resolve_shape_ref(self._shape_model['value'])
 
 
-class ServiceModel(object):
+class StringShape(Shape):
+    @CachedProperty
+    def enum(self):
+        return self.metadata.get('enum', [])
+
+
+class ServiceModel:
     """
 
     :ivar service_description: The parsed service description dictionary.
 
     """
-    # Any type not in this mapping will default to the Shape class.
-    SHAPE_CLASSES = {
-        'structure': StructureShape,
-        'list': ListShape,
-        'map': MapShape,
-    }
 
     def __init__(self, service_description, service_name=None):
         """
@@ -217,17 +299,45 @@ class ServiceModel(object):
         # We want clients to be able to access metadata directly.
         self.metadata = service_description.get('metadata', {})
         self._shape_resolver = ShapeResolver(
-            service_description.get('shapes', {}))
+            service_description.get('shapes', {})
+        )
         self._signature_version = NOT_SET
         self._service_name = service_name
+        self._instance_cache = {}
 
     def shape_for(self, shape_name, member_traits=None):
         return self._shape_resolver.get_shape_by_name(
-            shape_name, member_traits)
+            shape_name, member_traits
+        )
+
+    def shape_for_error_code(self, error_code):
+        return self._error_code_cache.get(error_code, None)
+
+    @CachedProperty
+    def _error_code_cache(self):
+        error_code_cache = {}
+        for error_shape in self.error_shapes:
+            code = error_shape.error_code
+            error_code_cache[code] = error_shape
+        return error_code_cache
 
     def resolve_shape_ref(self, shape_ref):
         return self._shape_resolver.resolve_shape_ref(shape_ref)
 
+    @CachedProperty
+    def shape_names(self):
+        return list(self._service_description.get('shapes', {}))
+
+    @CachedProperty
+    def error_shapes(self):
+        error_shapes = []
+        for shape_name in self.shape_names:
+            error_shape = self.shape_for(shape_name)
+            if error_shape.metadata.get('exception', False):
+                error_shapes.append(error_shape)
+        return error_shapes
+
+    @instance_cache
     def operation_model(self, operation_name):
         try:
             model = self._service_description['operations'][operation_name]
@@ -260,6 +370,13 @@ class ServiceModel(object):
         else:
             return self.endpoint_prefix
 
+    @CachedProperty
+    def service_id(self):
+        try:
+            return ServiceId(self._get_metadata_property('serviceId'))
+        except UndefinedModelAttributeError:
+            raise MissingServiceIdError(service_name=self._service_name)
+
     @CachedProperty
     def signing_name(self):
         """The name to use when computing signatures.
@@ -284,13 +401,31 @@ class ServiceModel(object):
     def endpoint_prefix(self):
         return self._get_metadata_property('endpointPrefix')
 
+    @CachedProperty
+    def endpoint_discovery_operation(self):
+        for operation in self.operation_names:
+            model = self.operation_model(operation)
+            if model.is_endpoint_discovery_operation:
+                return model
+
+    @CachedProperty
+    def endpoint_discovery_required(self):
+        for operation in self.operation_names:
+            model = self.operation_model(operation)
+            if (
+                model.endpoint_discovery is not None
+                and model.endpoint_discovery.get('required')
+            ):
+                return True
+        return False
+
     def _get_metadata_property(self, name):
         try:
             return self.metadata[name]
         except KeyError:
             raise UndefinedModelAttributeError(
-                '"%s" not defined in the metadata of the the model: %s' %
-                (name, self))
+                f'"{name}" not defined in the metadata of the model: {self}'
+            )
 
     # Signature version is one of the rare properties
     # than can be modified so a CachedProperty is not used here.
@@ -306,8 +441,11 @@ class ServiceModel(object):
     def signature_version(self, value):
         self._signature_version = value
 
+    def __repr__(self):
+        return f'{self.__class__.__name__}({self.service_name})'
 
-class OperationModel(object):
+
+class OperationModel:
     def __init__(self, operation_model, service_model, name=None):
         """
 
@@ -374,6 +512,20 @@ class OperationModel(object):
     def documentation(self):
         return self._operation_model.get('documentation', '')
 
+    @CachedProperty
+    def deprecated(self):
+        return self._operation_model.get('deprecated', False)
+
+    @CachedProperty
+    def endpoint_discovery(self):
+        # Explicit None default. An empty dictionary for this trait means it is
+        # enabled but not required to be used.
+        return self._operation_model.get('endpointdiscovery', None)
+
+    @CachedProperty
+    def is_endpoint_discovery_operation(self):
+        return self._operation_model.get('endpointoperation', False)
+
     @CachedProperty
     def input_shape(self):
         if 'input' not in self._operation_model:
@@ -381,7 +533,8 @@ class OperationModel(object):
             # input shape.
             return None
         return self._service_model.resolve_shape_ref(
-            self._operation_model['input'])
+            self._operation_model['input']
+        )
 
     @CachedProperty
     def output_shape(self):
@@ -391,22 +544,96 @@ class OperationModel(object):
             # operation has no expected output.
             return None
         return self._service_model.resolve_shape_ref(
-            self._operation_model['output'])
+            self._operation_model['output']
+        )
+
+    @CachedProperty
+    def idempotent_members(self):
+        input_shape = self.input_shape
+        if not input_shape:
+            return []
+
+        return [
+            name
+            for (name, shape) in input_shape.members.items()
+            if 'idempotencyToken' in shape.metadata
+            and shape.metadata['idempotencyToken']
+        ]
+
+    @CachedProperty
+    def auth_type(self):
+        return self._operation_model.get('authtype')
+
+    @CachedProperty
+    def error_shapes(self):
+        shapes = self._operation_model.get("errors", [])
+        return list(self._service_model.resolve_shape_ref(s) for s in shapes)
+
+    @CachedProperty
+    def endpoint(self):
+        return self._operation_model.get('endpoint')
+
+    @CachedProperty
+    def http_checksum_required(self):
+        return self._operation_model.get('httpChecksumRequired', False)
+
+    @CachedProperty
+    def http_checksum(self):
+        return self._operation_model.get('httpChecksum', {})
+
+    @CachedProperty
+    def has_event_stream_input(self):
+        return self.get_event_stream_input() is not None
+
+    @CachedProperty
+    def has_event_stream_output(self):
+        return self.get_event_stream_output() is not None
+
+    def get_event_stream_input(self):
+        return self._get_event_stream(self.input_shape)
+
+    def get_event_stream_output(self):
+        return self._get_event_stream(self.output_shape)
+
+    def _get_event_stream(self, shape):
+        """Returns the event stream member's shape if any or None otherwise."""
+        if shape is None:
+            return None
+        event_name = shape.event_stream_name
+        if event_name:
+            return shape.members[event_name]
+        return None
+
+    @CachedProperty
+    def has_streaming_input(self):
+        return self.get_streaming_input() is not None
 
     @CachedProperty
     def has_streaming_output(self):
-        output_shape = self.output_shape
-        if output_shape is None:
-            return False
-        payload = output_shape.serialization.get('payload')
+        return self.get_streaming_output() is not None
+
+    def get_streaming_input(self):
+        return self._get_streaming_body(self.input_shape)
+
+    def get_streaming_output(self):
+        return self._get_streaming_body(self.output_shape)
+
+    def _get_streaming_body(self, shape):
+        """Returns the streaming member's shape if any; or None otherwise."""
+        if shape is None:
+            return None
+        payload = shape.serialization.get('payload')
         if payload is not None:
-            payload_shape = output_shape.members[payload]
+            payload_shape = shape.members[payload]
             if payload_shape.type_name == 'blob':
-                return True
-        return False
+                return payload_shape
+        return None
+
+    def __repr__(self):
+        return f'{self.__class__.__name__}(name={self.name})'
 
 
-class ShapeResolver(object):
+class ShapeResolver:
     """Resolves shape references."""
 
     # Any type not in this mapping will default to the Shape class.
@@ -414,6 +641,7 @@ class ShapeResolver(object):
         'structure': StructureShape,
         'list': ListShape,
         'map': MapShape,
+        'string': StringShape,
     }
 
     def __init__(self, shape_map):
@@ -428,12 +656,14 @@ class ShapeResolver(object):
         try:
             shape_cls = self.SHAPE_CLASSES.get(shape_model['type'], Shape)
         except KeyError:
-            raise InvalidShapeError("Shape is missing required key 'type': %s"
-                                    % shape_model)
+            raise InvalidShapeError(
+                f"Shape is missing required key 'type': {shape_model}"
+            )
         if member_traits:
             shape_model = shape_model.copy()
             shape_model.update(member_traits)
-        return shape_cls(shape_name, shape_model, self)
+        result = shape_cls(shape_name, shape_model, self)
+        return result
 
     def resolve_shape_ref(self, shape_ref):
         # A shape_ref is a dict that has a 'shape' key that
@@ -452,23 +682,27 @@ class ShapeResolver(object):
                 shape_name = member_traits.pop('shape')
             except KeyError:
                 raise InvalidShapeReferenceError(
-                    "Invalid model, missing shape reference: %s" % shape_ref)
+                    f"Invalid model, missing shape reference: {shape_ref}"
+                )
             return self.get_shape_by_name(shape_name, member_traits)
 
 
-class UnresolvableShapeMap(object):
-    """A ShapeResolver that will throw ValueErrors when shapes are resolved.
-    """
+class UnresolvableShapeMap:
+    """A ShapeResolver that will throw ValueErrors when shapes are resolved."""
+
     def get_shape_by_name(self, shape_name, member_traits=None):
-        raise ValueError("Attempted to lookup shape '%s', but no shape "
-                         "map was provided.")
+        raise ValueError(
+            f"Attempted to lookup shape '{shape_name}', but no shape map was provided."
+        )
 
     def resolve_shape_ref(self, shape_ref):
-        raise ValueError("Attempted to resolve shape '%s', but no shape "
-                         "map was provided.")
+        raise ValueError(
+            f"Attempted to resolve shape '{shape_ref}', but no shape "
+            f"map was provided."
+        )
 
 
-class DenormalizedStructureBuilder(object):
+class DenormalizedStructureBuilder:
     """Build a StructureShape from a denormalized model.
 
     This is a convenience builder class that makes it easy to construct
@@ -504,6 +738,19 @@ class DenormalizedStructureBuilder(object):
                       matters, such as for documentation.
 
     """
+
+    SCALAR_TYPES = (
+        'string',
+        'integer',
+        'boolean',
+        'blob',
+        'float',
+        'timestamp',
+        'long',
+        'double',
+        'char',
+    )
+
     def __init__(self, name=None):
         self.members = OrderedDict()
         self._name_generator = ShapeNameGenerator()
@@ -536,9 +783,11 @@ class DenormalizedStructureBuilder(object):
         }
         self._build_model(denormalized, shapes, self.name)
         resolver = ShapeResolver(shape_map=shapes)
-        return StructureShape(shape_name=self.name,
-                              shape_model=shapes[self.name],
-                              shape_resolver=resolver)
+        return StructureShape(
+            shape_name=self.name,
+            shape_model=shapes[self.name],
+            shape_resolver=resolver,
+        )
 
     def _build_model(self, model, shapes, shape_name):
         if model['type'] == 'structure':
@@ -547,18 +796,17 @@ class DenormalizedStructureBuilder(object):
             shapes[shape_name] = self._build_list(model, shapes)
         elif model['type'] == 'map':
             shapes[shape_name] = self._build_map(model, shapes)
-        elif model['type'] in ['string', 'integer', 'boolean', 'blob', 'float',
-                               'timestamp', 'long', 'double', 'char']:
+        elif model['type'] in self.SCALAR_TYPES:
             shapes[shape_name] = self._build_scalar(model)
         else:
-            raise InvalidShapeError("Unknown shape type: %s" % model['type'])
+            raise InvalidShapeError(f"Unknown shape type: {model['type']}")
 
     def _build_structure(self, model, shapes):
         members = OrderedDict()
         shape = self._build_initial_shape(model)
         shape['members'] = members
 
-        for name, member_model in model['members'].items():
+        for name, member_model in model.get('members', OrderedDict()).items():
             member_shape_name = self._get_shape_name(member_model)
             members[name] = {'shape': member_shape_name}
             self._build_model(member_model, shapes, member_shape_name)
@@ -587,8 +835,9 @@ class DenormalizedStructureBuilder(object):
         }
         if 'documentation' in model:
             shape['documentation'] = model['documentation']
-        if 'enum' in model:
-            shape['enum'] = model['enum']
+        for attr in Shape.METADATA_ATTRS:
+            if attr in model:
+                shape[attr] = model[attr]
         return shape
 
     def _build_scalar(self, model):
@@ -601,13 +850,14 @@ class DenormalizedStructureBuilder(object):
             return self._name_generator.new_shape_name(model['type'])
 
 
-class ShapeNameGenerator(object):
+class ShapeNameGenerator:
     """Generate unique shape names for a type.
 
     This class can be used in conjunction with the DenormalizedStructureBuilder
     to generate unique shape names for a given type.
 
     """
+
     def __init__(self):
         self._name_cache = defaultdict(int)
 
@@ -639,5 +889,4 @@ class ShapeNameGenerator(object):
         """
         self._name_cache[type_name] += 1
         current_index = self._name_cache[type_name]
-        return '%sType%s' % (type_name.capitalize(),
-                             current_index)
+        return f'{type_name.capitalize()}Type{current_index}'
