@@ -38,7 +38,7 @@ def request_dict(**kwargs):
         'query_string': '',
         'method': 'POST',
         'url': 'https://example.com',
-        'context': {}
+        'context': {},
     }
     base.update(kwargs)
     return base
@@ -55,7 +55,6 @@ class RecordStreamResets(six.StringIO):
 
 
 class TestEndpointBase(unittest.TestCase):
-
     def setUp(self):
         self.op = mock.Mock()
         self.op.has_streaming_output = False
@@ -64,15 +63,19 @@ class TestEndpointBase(unittest.TestCase):
         self.event_emitter = mock.Mock()
         self.event_emitter.emit.return_value = []
         self.factory_patch = mock.patch(
-            'botocore.parsers.ResponseParserFactory')
+            'botocore.parsers.ResponseParserFactory'
+        )
         self.factory = self.factory_patch.start()
         self.endpoint = Endpoint(
             'https://ec2.us-west-2.amazonaws.com/',
             endpoint_prefix='ec2',
-            event_emitter=self.event_emitter)
+            event_emitter=self.event_emitter,
+        )
         self.http_session = mock.Mock()
         self.http_session.send.return_value = mock.Mock(
-            status_code=200, headers={}, content=b'{"Foo": "bar"}',
+            status_code=200,
+            headers={},
+            content=b'{"Foo": "bar"}',
         )
         self.endpoint.http_session = self.http_session
 
@@ -102,7 +105,6 @@ class TestEndpointBase(unittest.TestCase):
 
 
 class TestEndpointFeatures(TestEndpointBase):
-
     def test_make_request_with_no_auth(self):
         self.endpoint.auth = None
         self.endpoint.make_request(self.op, request_dict())
@@ -123,7 +125,9 @@ class TestEndpointFeatures(TestEndpointBase):
     def test_make_request_with_context(self):
         r = request_dict()
         r['context'] = {'signing': {'region': 'us-west-2'}}
-        with mock.patch('botocore.endpoint.Endpoint.prepare_request') as prepare:
+        with mock.patch(
+            'botocore.endpoint.Endpoint.prepare_request'
+        ) as prepare:
             self.endpoint.make_request(self.op, r)
         request = prepare.call_args[0][0]
         self.assertEqual(request.context['signing']['region'], 'us-west-2')
@@ -131,7 +135,9 @@ class TestEndpointFeatures(TestEndpointBase):
     def test_make_request_sets_retries_config_in_context(self):
         r = request_dict()
         r['context'] = {'signing': {'region': 'us-west-2'}}
-        with mock.patch('botocore.endpoint.Endpoint.prepare_request') as prepare:
+        with mock.patch(
+            'botocore.endpoint.Endpoint.prepare_request'
+        ) as prepare:
             self.endpoint.make_request(self.op, r)
         request = prepare.call_args[0][0]
         self.assertIn('retries', request.context)
@@ -139,11 +145,14 @@ class TestEndpointFeatures(TestEndpointBase):
     def test_exception_caught_when_constructing_retries_context(self):
         r = request_dict()
         datetime_patcher = mock.patch.object(
-            botocore.endpoint.datetime, 'datetime',
-            mock.Mock(wraps=datetime.datetime)
+            botocore.endpoint.datetime,
+            'datetime',
+            mock.Mock(wraps=datetime.datetime),
         )
         r['context'] = {'signing': {'region': 'us-west-2'}}
-        with mock.patch('botocore.endpoint.Endpoint.prepare_request') as prepare:
+        with mock.patch(
+            'botocore.endpoint.Endpoint.prepare_request'
+        ) as prepare:
             mocked_datetime = datetime_patcher.start()
             mocked_datetime.side_effect = Exception()
             self.endpoint.make_request(self.op, r)
@@ -161,7 +170,9 @@ class TestEndpointFeatures(TestEndpointBase):
 
         r = request_dict()
         self.http_session.send.return_value = mock.Mock(
-            status_code=400, headers={}, content=b'',
+            status_code=400,
+            headers={},
+            content=b'',
         )
         parser = mock.Mock()
         parser.parse.side_effect = [
@@ -193,7 +204,7 @@ class TestEndpointFeatures(TestEndpointBase):
 
 class TestRetryInterface(TestEndpointBase):
     def setUp(self):
-        super(TestRetryInterface, self).setUp()
+        super().setUp()
         self.retried_on_exception = None
         self._operation = mock.Mock(spec=OperationModel)
         self._operation.name = 'DescribeInstances'
@@ -204,17 +215,18 @@ class TestRetryInterface(TestEndpointBase):
 
     def assert_events_emitted(self, event_emitter, expected_events):
         self.assertEqual(
-            self.get_events_emitted(event_emitter), expected_events)
+            self.get_events_emitted(event_emitter), expected_events
+        )
 
     def test_retry_events_are_emitted(self):
         self.endpoint.make_request(self._operation, request_dict())
         call_args = self.event_emitter.emit.call_args
-        self.assertEqual(call_args[0][0],
-                         'needs-retry.ec2.DescribeInstances')
+        self.assertEqual(call_args[0][0], 'needs-retry.ec2.DescribeInstances')
 
     def test_retry_events_can_alter_behavior(self):
         self.event_emitter.emit.side_effect = self.get_emitter_responses(
-            num_retries=1)
+            num_retries=1
+        )
         r = request_dict()
         r['context']['client_config'] = Config()
         self.endpoint.make_request(self._operation, r)
@@ -225,12 +237,14 @@ class TestRetryInterface(TestEndpointBase):
                 'before-send.ec2.DescribeInstances',
                 'response-received.ec2.DescribeInstances',
                 'needs-retry.ec2.DescribeInstances',
-            ] * 2
+            ]
+            * 2,
         )
 
     def test_retry_on_socket_errors(self):
         self.event_emitter.emit.side_effect = self.get_emitter_responses(
-            num_retries=1)
+            num_retries=1
+        )
         self.http_session.send.side_effect = HTTPClientError(error='wrapped')
         with self.assertRaises(HTTPClientError):
             self.endpoint.make_request(self._operation, request_dict())
@@ -241,12 +255,14 @@ class TestRetryInterface(TestEndpointBase):
                 'before-send.ec2.DescribeInstances',
                 'response-received.ec2.DescribeInstances',
                 'needs-retry.ec2.DescribeInstances',
-            ] * 2
+            ]
+            * 2,
         )
 
     def test_retry_attempts_added_to_response_metadata(self):
         self.event_emitter.emit.side_effect = self.get_emitter_responses(
-            num_retries=1)
+            num_retries=1
+        )
         parser = mock.Mock()
         parser.parse.return_value = {'ResponseMetadata': {}}
         self.factory.return_value.create_parser.return_value = parser
@@ -257,7 +273,8 @@ class TestRetryInterface(TestEndpointBase):
 
     def test_retry_attempts_is_zero_when_not_retried(self):
         self.event_emitter.emit.side_effect = self.get_emitter_responses(
-            num_retries=0)
+            num_retries=0
+        )
         parser = mock.Mock()
         parser.parse.return_value = {'ResponseMetadata': {}}
         self.factory.return_value.create_parser.return_value = parser
@@ -267,7 +284,7 @@ class TestRetryInterface(TestEndpointBase):
 
 class TestS3ResetStreamOnRetry(TestEndpointBase):
     def setUp(self):
-        super(TestS3ResetStreamOnRetry, self).setUp()
+        super().setUp()
 
     def max_attempts_retry_handler(self, attempts, **kwargs):
         # Simulate a max requests of 3.
@@ -298,7 +315,6 @@ class TestS3ResetStreamOnRetry(TestEndpointBase):
 
 
 class TestEventStreamBody(TestEndpointBase):
-
     def test_event_stream_body_is_streaming(self):
         self.op.has_event_stream_output = True
         request = request_dict()
@@ -310,8 +326,8 @@ class TestEventStreamBody(TestEndpointBase):
 class TestEndpointCreator(unittest.TestCase):
     def setUp(self):
         self.service_model = mock.Mock(
-            endpoint_prefix='ec2', signature_version='v2',
-            signing_name='ec2')
+            endpoint_prefix='ec2', signature_version='v2', signing_name='ec2'
+        )
         self.environ = {}
         self.environ_patch = mock.patch('os.environ', self.environ)
         self.environ_patch.start()
@@ -323,100 +339,129 @@ class TestEndpointCreator(unittest.TestCase):
 
     def test_creates_endpoint_with_configured_url(self):
         endpoint = self.creator.create_endpoint(
-            self.service_model, region_name='us-east-1',
-            endpoint_url='https://endpoint.url')
+            self.service_model,
+            region_name='us-east-1',
+            endpoint_url='https://endpoint.url',
+        )
         self.assertEqual(endpoint.host, 'https://endpoint.url')
 
     def test_creates_endpoint_with_ipv4_url(self):
         endpoint = self.creator.create_endpoint(
-            self.service_model, region_name='us-east-1',
-            endpoint_url='https://192.168.0.0')
+            self.service_model,
+            region_name='us-east-1',
+            endpoint_url='https://192.168.0.0',
+        )
         self.assertEqual(endpoint.host, 'https://192.168.0.0')
 
     def test_creates_endpoint_with_ipv6_url(self):
         endpoint = self.creator.create_endpoint(
-            self.service_model, region_name='us-east-1',
-            endpoint_url='https://[100:0:2::61]:7480')
+            self.service_model,
+            region_name='us-east-1',
+            endpoint_url='https://[100:0:2::61]:7480',
+        )
         self.assertEqual(endpoint.host, 'https://[100:0:2::61]:7480')
 
     def test_raises_error_with_invalid_url(self):
         with pytest.raises(ValueError):
             self.creator.create_endpoint(
-                self.service_model, region_name='us-east-1',
-                endpoint_url='https://*.aws.amazon.com/'
+                self.service_model,
+                region_name='us-east-1',
+                endpoint_url='https://*.aws.amazon.com/',
             )
 
     def test_create_endpoint_with_default_timeout(self):
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
+            self.service_model,
+            region_name='us-west-2',
             endpoint_url='https://example.com',
-            http_session_cls=self.mock_session)
+            http_session_cls=self.mock_session,
+        )
         session_args = self.mock_session.call_args[1]
         self.assertEqual(session_args.get('timeout'), DEFAULT_TIMEOUT)
 
     def test_create_endpoint_with_customized_timeout(self):
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
-            endpoint_url='https://example.com', timeout=123,
-            http_session_cls=self.mock_session)
+            self.service_model,
+            region_name='us-west-2',
+            endpoint_url='https://example.com',
+            timeout=123,
+            http_session_cls=self.mock_session,
+        )
         session_args = self.mock_session.call_args[1]
         self.assertEqual(session_args.get('timeout'), 123)
 
     def test_get_endpoint_default_verify_ssl(self):
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
+            self.service_model,
+            region_name='us-west-2',
             endpoint_url='https://example.com',
-            http_session_cls=self.mock_session)
+            http_session_cls=self.mock_session,
+        )
         session_args = self.mock_session.call_args[1]
         self.assertTrue(session_args.get('verify'))
 
     def test_verify_ssl_can_be_disabled(self):
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
-            endpoint_url='https://example.com', verify=False,
-            http_session_cls=self.mock_session)
+            self.service_model,
+            region_name='us-west-2',
+            endpoint_url='https://example.com',
+            verify=False,
+            http_session_cls=self.mock_session,
+        )
         session_args = self.mock_session.call_args[1]
         self.assertFalse(session_args.get('verify'))
 
     def test_verify_ssl_can_specify_cert_bundle(self):
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
-            endpoint_url='https://example.com', verify='/path/cacerts.pem',
-            http_session_cls=self.mock_session)
+            self.service_model,
+            region_name='us-west-2',
+            endpoint_url='https://example.com',
+            verify='/path/cacerts.pem',
+            http_session_cls=self.mock_session,
+        )
         session_args = self.mock_session.call_args[1]
         self.assertEqual(session_args.get('verify'), '/path/cacerts.pem')
 
     def test_client_cert_can_specify_path(self):
         client_cert = '/some/path/cert'
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
-            endpoint_url='https://example.com', client_cert=client_cert,
-            http_session_cls=self.mock_session)
+            self.service_model,
+            region_name='us-west-2',
+            endpoint_url='https://example.com',
+            client_cert=client_cert,
+            http_session_cls=self.mock_session,
+        )
         session_args = self.mock_session.call_args[1]
         self.assertEqual(session_args.get('client_cert'), '/some/path/cert')
 
     def test_honor_cert_bundle_env_var(self):
         self.environ['REQUESTS_CA_BUNDLE'] = '/env/cacerts.pem'
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
+            self.service_model,
+            region_name='us-west-2',
             endpoint_url='https://example.com',
-            http_session_cls=self.mock_session)
+            http_session_cls=self.mock_session,
+        )
         session_args = self.mock_session.call_args[1]
         self.assertEqual(session_args.get('verify'), '/env/cacerts.pem')
 
     def test_env_ignored_if_explicitly_passed(self):
         self.environ['REQUESTS_CA_BUNDLE'] = '/env/cacerts.pem'
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
-            endpoint_url='https://example.com', verify='/path/cacerts.pem',
-            http_session_cls=self.mock_session)
+            self.service_model,
+            region_name='us-west-2',
+            endpoint_url='https://example.com',
+            verify='/path/cacerts.pem',
+            http_session_cls=self.mock_session,
+        )
         session_args = self.mock_session.call_args[1]
         # /path/cacerts.pem wins over the value from the env var.
         self.assertEqual(session_args.get('verify'), '/path/cacerts.pem')
 
     def test_can_specify_max_pool_conns(self):
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
+            self.service_model,
+            region_name='us-west-2',
             endpoint_url='https://example.com',
             max_pool_connections=100,
             http_session_cls=self.mock_session,
@@ -427,8 +472,11 @@ class TestEndpointCreator(unittest.TestCase):
     def test_socket_options(self):
         socket_options = [(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)]
         self.creator.create_endpoint(
-            self.service_model, region_name='us-west-2',
+            self.service_model,
+            region_name='us-west-2',
             endpoint_url='https://example.com',
-            http_session_cls=self.mock_session, socket_options=socket_options)
+            http_session_cls=self.mock_session,
+            socket_options=socket_options,
+        )
         session_args = self.mock_session.call_args[1]
         self.assertEqual(session_args.get('socket_options'), socket_options)
